@@ -36,6 +36,11 @@
 #include <checkers.h>
 #include <path_state.h>
 #include <safe_printf.h>
+#include <blacklist.h>
+#include <hwtable.h>
+#include <util.h>
+#include <defaults.h>
+#include <structs.h>
 
 #include "main.h"
 #include "devinfo.h"
@@ -58,18 +63,6 @@ filepresent (char * run) {
 	return 0;
 }
 
-static int
-blacklist (char * dev) {
-	int i;
-	char *p;
-
-	vector_foreach_slot (conf->blist, p, i)
-		if (memcmp(dev, p, strlen(p)) == 0)
-			return 1;
-	
-	return 0;
-}
-
 static int
 get_pathvec_sysfs (vector pathvec)
 {
@@ -133,7 +126,7 @@ get_pathvec_sysfs (vector pathvec)
 	sysfs_read_directory(sdir);
 
 	dlist_for_each_data(sdir->subdirs, devp, struct sysfs_directory) {
-		if (blacklist(devp->name))
+		if (blacklist(conf->blist, devp->name))
 			continue;
 
 		sysfs_read_directory(devp);
@@ -157,6 +150,7 @@ get_pathvec_sysfs (vector pathvec)
 		}
 		if (devinfo(curpath)) {
 			free (curpath);
+			fprintf(stderr, "pb getting path info, free\n");
 			continue;
 		}
 		if (memcmp(empty_buff, refwwid, WWID_SIZE) != 0 && 
@@ -237,7 +231,7 @@ print_map (struct multipath * mpp)
 {
 	if (mpp->size && mpp->params)
 		printf("0 %lu %s %s\n",
-			 mpp->size, DM_TARGET, mpp->params);
+			 mpp->size, DEFAULT_TARGET, mpp->params);
 	return;
 }
 
@@ -278,7 +272,7 @@ print_mp (struct multipath * mpp)
 	struct path * pp = NULL;
 	struct pathgroup * pgp = NULL;
 
-	if (mpp->action == ACT_NOTHING)
+	if (mpp->action == ACT_NOTHING || conf->verbosity == 0)
 		return;
 
 	if (conf->verbosity > 1) {
@@ -368,6 +362,9 @@ print_all_mp (vector mp)
 	int k;
 	struct multipath * mpp;
 
+	if (conf->verbosity == 0)
+		return;
+
 	vector_foreach_slot (mp, mpp, k) 
 		print_mp(mpp);
 }
@@ -403,7 +400,8 @@ coalesce_paths (vector mp, vector pathvec)
 		mpp->size = pp1->size;
 
 		mpp->mpe = find_mp(pp1->wwid);
-		mpp->hwe = find_hw(pp1->vendor_id, pp1->product_id);
+		mpp->hwe = find_hw(conf->hwtable,
+				   pp1->vendor_id, pp1->product_id);
 
 		mpp->paths = vector_alloc();
 		vector_alloc_slot (mpp->paths);
@@ -737,7 +735,7 @@ select_action (struct multipath * mpp, vector curmp)
 			return;
 		}
 		if (VECTOR_SIZE(cmpp->pg) != VECTOR_SIZE(mpp->pg)) {
-			dbg("different number og PG");
+			dbg("different number of PG");
 			mpp->action = ACT_RELOAD;
 			return;
 		}
@@ -817,22 +815,21 @@ domap (struct multipath * mpp)
 	 */
 	dm_log_init_verbose(0);
 
-	if (op == DM_DEVICE_RELOAD) {
-		if (!dm_simplecmd(DM_DEVICE_SUSPEND, mpp->alias))
-			goto out;
-	}
+	r = dm_addmap(op, mpp->alias, DEFAULT_TARGET, mpp->params, mpp->size);
 
-	if (!dm_addmap(op, mpp->alias, DM_TARGET, mpp->params, mpp->size)) {
+	if (r == 0)
 		dm_simplecmd(DM_DEVICE_REMOVE, mpp->alias);
-		goto out;
-	}
+	else if (op == DM_DEVICE_RELOAD)
+		dm_simplecmd(DM_DEVICE_RESUME, mpp->alias);
 
-	if (op == DM_DEVICE_RELOAD)
-		if (!dm_simplecmd(DM_DEVICE_RESUME, mpp->alias))
-			goto out;
+	/*
+	 * PG order is random, so we need to set the primary one
+	 * upon create or reload
+	 */
+	dm_switchgroup(mpp->alias, mpp->nextpg);
 
-	out:
 	dm_log_init_verbose(1);
+
 	return r;
 }
 
@@ -861,7 +858,7 @@ dm_get_maps (vector mp, char * type)
 		goto out;
 
 	do {
-		if (dm_type(names->name, DM_TARGET)) {
+		if (dm_type(names->name, DEFAULT_TARGET)) {
 			dm_get_map(names->name, &length, &params);
 			dm_get_status(names->name, &status);
 			mpp = zalloc(sizeof(struct multipath));
@@ -898,7 +895,7 @@ signal_daemon (void)
 
 	buf = malloc (8);
 
-	file = fopen (PIDFILE, "r");
+	file = fopen (DEFAULT_PIDFILE, "r");
 
 	if (!file) {
 		if (conf->verbosity > 0)
@@ -916,69 +913,6 @@ signal_daemon (void)
 	kill (pid, SIGHUP);
 }
 
-#define VECTOR_ADDSTR(a, b) \
-	str = zalloc (6 * sizeof(char)); \
-	snprintf (str, 6, b); \
-	vector_alloc_slot(a); \
-	vector_set_slot(a, str);
-
-static void
-setup_default_blist (vector blist)
-{
-	char * str;
-	
-	VECTOR_ADDSTR(blist, "cciss");
-	VECTOR_ADDSTR(blist, "fd");
-	VECTOR_ADDSTR(blist, "hd");
-	VECTOR_ADDSTR(blist, "md");
-	VECTOR_ADDSTR(blist, "dm");
-	VECTOR_ADDSTR(blist, "sr");
-	VECTOR_ADDSTR(blist, "scd");
-	VECTOR_ADDSTR(blist, "st");
-	VECTOR_ADDSTR(blist, "ram");
-	VECTOR_ADDSTR(blist, "raw");
-	VECTOR_ADDSTR(blist, "loop");
-}
-
-#define VECTOR_ADDHWE(a, b, c, d, e) \
-	hwe = zalloc (sizeof(struct hwentry)); \
-	hwe->vendor = zalloc (SCSI_VENDOR_SIZE * sizeof(char)); \
-	snprintf (hwe->vendor, SCSI_VENDOR_SIZE, b); \
-	hwe->product = zalloc (SCSI_PRODUCT_SIZE * sizeof(char)); \
-	snprintf (hwe->product, SCSI_PRODUCT_SIZE, c); \
-	hwe->pgpolicy = d; \
-	hwe->getuid = e; \
-	vector_alloc_slot(a); \
-	vector_set_slot(a, hwe);
-	
-static void
-setup_default_hwtable (vector hwtable)
-{
-	struct hwentry * hwe;
-	
-	VECTOR_ADDHWE(hwtable, "COMPAQ", "HSV110 (C)COMPAQ", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "COMPAQ", "MSA1000", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "COMPAQ", "MSA1000 VOLUME", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "DEC", "HSG80", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "HP", "HSV110", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "HP", "A6189A", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "HP", "OPEN-", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "DDN", "SAN DataDirector", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "FSC", "CentricStor", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "HITACHI", "DF400", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "HITACHI", "DF500", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "HITACHI", "DF600", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "IBM", "ProFibre 4000R", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "SGI", "TP9100", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "SGI", "TP9300", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "SGI", "TP9400", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "SGI", "TP9500", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "3PARdata", "VV", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "STK", "OPENstorage D280", GROUP_BY_SERIAL, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "SUN", "StorEdge 3510", MULTIBUS, "/bin/scsi_id -g -s");
-	VECTOR_ADDHWE(hwtable, "SUN", "T4", MULTIBUS, "/bin/scsi_id -g -s");
-}
-
 static void
 usage (char * progname)
 {
@@ -1055,16 +989,16 @@ main (int argc, char *argv[])
 	extern char *optarg;
 	extern int optind;
 
-	if (dm_prereq(DM_TARGET, 1, 0, 3)) {
+	if (dm_prereq(DEFAULT_TARGET, 1, 0, 3)) {
 		fprintf(stderr, "device mapper prerequisites not met.\n");
 		exit(1);
 	}
 
 	/*
 	 * Don't run in parallel
-	 * ie, acquire a F_WRLCK-type lock on RUN
+	 * ie, acquire a F_WRLCK-type lock on DEFAULT_RUNFILE
 	 */
-	if (try_lock(RUN)) {
+	if (try_lock(DEFAULT_RUNFILE)) {
 		fprintf(stderr, "waited for to long. exiting\n");
 		exit(1);
 	}
@@ -1108,11 +1042,12 @@ main (int argc, char *argv[])
 			conf->signal = 0;
 			break;
 		case 'F':
-			dm_flush_maps(DM_TARGET);
+			dm_flush_maps(DEFAULT_TARGET);
 			goto out;
 			break;
 		case 'l':
 			conf->list = 1;
+			conf->dry_run = 1;
 			conf->signal = 0;
 			break;
 		case 'S':
@@ -1155,8 +1090,8 @@ main (int argc, char *argv[])
 	/*
 	 * read the config file
 	 */
-	if (filepresent(CONFIGFILE))
-		init_data (CONFIGFILE, init_keywords);
+	if (filepresent(DEFAULT_CONFIGFILE))
+		init_data (DEFAULT_CONFIGFILE, init_keywords);
 	
 	/*
 	 * fill the voids left in the config file
@@ -1187,13 +1122,19 @@ main (int argc, char *argv[])
 	if (conf->default_hwhandler == NULL)
 		conf->default_hwhandler = DEFAULT_HWHANDLER;
 
+	/*
+	 * if we have a blacklisted device parameterexit early
+	 */
+	if (conf->dev && blacklist(conf->blist, conf->dev))
+		exit(0);
+	
 	/*
 	 * get a path list and group them as multipaths
 	 */
-	if (get_pathvec_sysfs(pathvec))
+	if (get_pathvec_sysfs(pathvec) || VECTOR_SIZE(pathvec) == 0)
 		exit(1);
 
-	dm_get_maps(curmp, DM_TARGET);
+	dm_get_maps(curmp, DEFAULT_TARGET);
 
 	if (VECTOR_SIZE(pathvec) == 0 && conf->verbosity > 0) {
 		fprintf(stdout, "no path found\n");
@@ -1227,7 +1168,7 @@ main (int argc, char *argv[])
 		    0 == strncmp(mpp->wwid, conf->dev, FILE_NAME_SIZE))) {
 			setup_map(mpp);
 
-			if (!mpp->action)
+			if (mpp->action == 0)
 				select_action(mpp, curmp);
 			domap(mpp);
 			goto out;
@@ -1237,7 +1178,7 @@ main (int argc, char *argv[])
 	vector_foreach_slot (mp, mpp, k) {
 		setup_map(mpp);
 
-		if (!mpp->action)
+		if (mpp->action == 0)
 			select_action(mpp, curmp);
 		domap(mpp);
 	}
